@@ -8,9 +8,10 @@ DATABASES = {
 #        'TEST_NAME': '/tmp/chroniker.db',
     }
 }
-#ROOT_URLCONF = 'chroniker.urls'
+ROOT_URLCONF = 'chroniker.tests.urls'
 INSTALLED_APPS = [
     'django.contrib.auth',
+    'django.contrib.admin',
     'django.contrib.contenttypes',
     'django.contrib.sessions',
     'django.contrib.sites',
@@ -23,4 +24,9 @@ USE_TZ = True
 
 AUTH_USER_MODEL = 'auth.User'
 
-SECRET_KEY = 'abc123'
\ No newline at end of file
+SECRET_KEY = 'abc123'
+
+SITE_ID = 1
+
+BASE_SECURE_URL = 'https://localhost'
+BASE_URL = 'http://localhost'
